/* Operating system specific defines to be used when targeting GCC for
   hosting on Windows32, using GNU tools and the Windows32 API Library,
   as distinct from winnt.h, which is used to build GCC for use with a
   windows style library and tool set and uses the Microsoft tools.
   Copyright (C) 1997, 1998, 1999, 2003 Free Software Foundation, Inc.

This file is part of GNU CC.

GNU CC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU CC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU CC; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA. */

/* Most of this is the same as for cygwin, except for changing some specs.  */
#ifndef NATIVE_MSYS
/* this is to quard against some undefs of command-line defines by cygwin.h */
#define NATIVE_MSYS
#endif

/* Please keep changes to CPP_PREDEFINES in sync with i386/crtdll. The
   only difference between the two should be __MSVCRT__ needed to 
   distinguish MSVC from CRTDLL runtime in mingw headers. */
#undef CPP_PREDEFINES
#define CPP_PREDEFINES "-D_WIN32 -D__WIN32 -D__WIN32__ -DWIN32 \
  -DWINNT -D_X86_=1 \
  -Asystem=winnt"

/* Specific a different directory for the standard include files.  */
#undef STANDARD_INCLUDE_DIR
#define STANDARD_INCLUDE_DIR "/usr/local/i686-pc-msys/include"

#define STANDARD_INCLUDE_COMPONENT "MSYS"

#undef CPP_SPEC
#define CPP_SPEC \
  "%(cpp_cpu) %{posix:-D_POSIX_SOURCE} \
  -D__stdcall=__attribute__((__stdcall__)) \
  -D__cdecl=__attribute__((__cdecl__)) \
  -D__fastcall=__attribute__((__fastcall__)) \
   %{mno-cygwin:-D__MSVCRT__ -D__MINGW32__ %{!ansi:%{mthreads:-D_MT}}} \
   %{!mno-cygwin:-D__MSYS__ %{!ansi:-Dunix -D_unix} -D__unix__ -D__unix } \
   %{!ansi:-D_stdcall=__attribute__((__stdcall__)) \
   -D_cdecl=__attribute__((__cdecl__)) \
   -D_fastcall=__attribute__((__fastcall__))} \
  -D__declspec(x)=__attribute__((x)) \
  %{!mno-cygwin: -isystem /usr/local/include -idirafter /usr/include} \
  %{mno-cygwin: -isystem /mingw/include -isystem /usr/mingw/include -isystem /usr/include/mingw32} \
  %{!nostdinc:%{!mno-win32|mno-cygwin: -idirafter /usr/include/w32api%s \
  -idirafter /usr/local/include/w32api%s -idirafter ../include/w32api%s \
  -idirafter ../../include/w32api%s}}"


/* Override Cygwin definition, which is conditional on -mno-cygwin  */
#undef CPLUSPLUS_CPP_SPEC

/* For Windows applications, include more libraries, but always include
   kernel32.  */
#undef LIB_SPEC
#define LIB_SPEC \
   "%{pg:-lgmon} \
   %{mwindows:-lgdi32 -lcomdlg32} \
   -luser32 -lkernel32 -ladvapi32 -lshell32"

#undef LINK_SPEC
#define LINK_SPEC \
  "%{mwindows:--subsystem windows} \
  %{mconsole:--subsystem console} \
  %{shared: %{mdll: %eshared and mdll are not compatible}} \
  %{shared: --shared --enable-auto-image-base} \
  %{mdll:--dll} \
  %{static:-Bstatic} %{!static:-Bdynamic} \
  %{shared|mdll: -e \
  %{mno-cygwin:_DllMainCRTStartup@12} \
  %{!mno-cygwin:__msys_dll_entry@12}}\
  %{!mno-cygwin:--dll-search-prefix=msys}"

/* Include libgcc */
#undef LIBGCC_SPEC
#define LIBGCC_SPEC \
 "%{mno-cygwin: %{mthreads:-lmingwthrd} -lmingw32 -lgcc -lmoldname -lmingwex -lmsvcrt} \
  %{!mno-cygwin:-lmsys-1.0 -lgcc}"

#undef STARTFILE_SPEC
#define STARTFILE_SPEC \
 "%{mno-cygwin: %{shared|mdll:dllcrt2%O%s} %{!shared:%{!mdll:crt2%O%s}} %{pg:gcrt2%O%s} crtbegin%O%s} \
  %{!mno-cygwin: %{!shared: %{!mdll:crt0%O%s %{pg:gcrt0%O%s}}}}"

#undef ENDFILE_SPEC
#define ENDFILE_SPEC "%{mno-cygwin:crtend%O%s}"

/* Output STRING, a string representing a filename, to FILE.  We canonicalize
   it to be in MS-DOS format.  */
#define OUTPUT_QUOTED_STRING(FILE, STRING) \
do {						\
  char c;					\
						\
  putc ('\"', asm_file);			\
  if (STRING[1] == ':'				\
      && (STRING[2] == '/' || STRING[2] == '\\')) \
    {						\
      putc ('/', asm_file);			\
      putc ('/', asm_file);			\
      putc (*string, asm_file);			\
      string += 2;				\
    }						\
						\
  while ((c = *string++) != 0)			\
    {						\
      if (c == '\\')				\
	c = '/';				\
						\
      if (c == '\"')				\
	putc ('\\', asm_file);			\
      putc (c, asm_file);			\
    }						\
						\
  putc ('\"', asm_file);			\
} while (0)

/* Override Cygwin's definition. This is necessary now due to the way
   Cygwin profiling code is written. Once "fixed", we can remove this.  */
#undef SUBTARGET_PROLOGUE
